#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <stdio.h>
#include <unistd.h>

static char *const rcinitcmd[]     = { "/etc/rc", NULL };
static char *const rchaltcmd[]     = { "/etc/rc.shutdown", "halt", NULL };
static char *const rcpoweroffcmd[] = { "/etc/rc.shutdown", "poweroff", NULL };
static char *const rcrebootcmd[]   = { "/etc/rc.shutdown", "reboot", NULL };

static sigset_t set;

static void spawn(char *const argv[])
{
	int child_pid = fork();
	if (child_pid == -1) perror("fork");
	else if (!child_pid) {
		sigprocmask(SIG_UNBLOCK, &set, NULL);
		setsid();
		execvp(argv[0], argv);
		perror("execvp");
		_exit(1);
	}
}

static void handle(int sig) {
	if      (sig == SIGUSR1) spawn(rchaltcmd);
	else if (sig == SIGUSR2) spawn(rcpoweroffcmd);
	else if (sig == SIGINT || sig == SIGTERM) spawn(rcrebootcmd);
	else if (sig == SIGCHLD) while (waitpid(-1, NULL, WNOHANG) > 0) ;
}

int main(void)
{
	int sig;
	if (getpid() != 1)
		return 1;
	chdir("/");
	sigfillset(&set);
	sigprocmask(SIG_BLOCK, &set, NULL);
	spawn(rcinitcmd);
	while (1) {
		sigwait(&set, &sig);
		handle(sig);
	}
}
